package org.briarproject.bramble.db;

import org.briarproject.bramble.api.db.DbException;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.logging.Logger;

import javax.annotation.Nullable;

import static java.util.logging.Level.WARNING;
import static org.briarproject.bramble.util.LogUtils.logException;

class Migration39_40 implements Migration<Connection> {

	private static final Logger LOG =
			Logger.getLogger(Migration39_40.class.getName());

	@Override
	public int getStartVersion() {
		return 39;
	}

	@Override
	public int getEndVersion() {
		return 40;
	}

	@Override
	public void migrate(Connection txn) throws DbException {
		Statement s = null;
		try {
			s = txn.createStatement();
			// Add type column to contacts
			s.execute("ALTER TABLE contacts"
					+ " ADD type INT");
			// Set type to CONTACT (0)
			s.execute("UPDATE contacts"
					+ " SET type = 0");
			// Set type NOT NULL
			s.execute("ALTER TABLE contacts"
					+ " ALTER COLUMN type INT NOT NULL");
			s.execute("CREATE TABLE mailboxes"
					+ " (contactId INT NOT NULL,"
					+ " mailboxId INT,"
					+ " aliasId INT,"
					+ " PRIMARY KEY (contactId),"
					+ " FOREIGN KEY (contactId)"
					+ " REFERENCES contacts (contactId)"
					+ " ON DELETE CASCADE,"
					+ " FOREIGN KEY (mailboxId)"
					+ " REFERENCES contacts (contactId)"
					+ " ON DELETE CASCADE)");
		} catch (SQLException e) {
			tryToClose(s);
			throw new DbException(e);
		}
	}

	private void tryToClose(@Nullable Statement s) {
		try {
			if (s != null) s.close();
		} catch (SQLException e) {
			logException(LOG, WARNING, e);
		}
	}
}
