package org.briarproject.bramble.mailbox;

import org.briarproject.bramble.api.contact.ContactId;
import org.briarproject.bramble.api.sync.SyncSession;
import org.briarproject.bramble.api.sync.SyncSessionFactory;
import org.briarproject.bramble.api.transport.KeyManager;
import org.briarproject.bramble.api.transport.StreamReaderFactory;
import org.briarproject.bramble.api.transport.StreamWriterFactory;
import org.briarproject.bramble.mailbox.protocol.MailboxProtocol;
import org.briarproject.bramble.mailbox.protocol.MailboxRequestTake;

import java.io.IOException;
import java.util.concurrent.Executor;
import java.util.logging.Logger;

class MailboxOwnerSession extends AbstractMailboxSession {

	private static final Logger LOG =
			Logger.getLogger(MailboxOwnerSession.class.getName());
	private final Executor ioExecutor;
	private final MailboxProtocol mailboxProtocol;
	private SyncSession duplexOutgoingSession;

	public MailboxOwnerSession(ContactId contactId, Executor ioExecutor,
			KeyManager keyManager,
			SyncSessionFactory syncSessionFactory,
			StreamWriterFactory streamWriterFactory,
			StreamReaderFactory streamReaderFactory,
			MailboxProtocol mailboxProtocol, int transportMaxLatency,
			int transportMaxIdleTime) {

		super(ioExecutor, keyManager, syncSessionFactory, streamWriterFactory,
				streamReaderFactory, mailboxProtocol, transportMaxLatency,
				transportMaxIdleTime, contactId);

		this.ioExecutor = ioExecutor;
		this.mailboxProtocol = mailboxProtocol;
	}

	@Override
	public void run() throws IOException {
		ioExecutor.execute(() -> readRequests());
		duplexOutgoingSession = createDuplexOutgoingSession();
		duplexOutgoingSession.run();
		try {
			endSession();
		} catch (InterruptedException e) {
			throw new IOException("Interrupted while ending session");
		}
	}

	@Override
	protected void handleProtocolException() {
		if (duplexOutgoingSession != null)
			duplexOutgoingSession.interrupt();
	}

	@Override
	public void handleTake(MailboxRequestTake mailboxRequestTake)
			throws MailboxSessionHandleException {
		throw new MailboxSessionHandleException(
				"MailboxOwnerSession does not support the TAKE request");
	}
}
