package org.briarproject.bramble.mailbox;

import org.briarproject.bramble.api.contact.ContactId;
import org.briarproject.bramble.api.contact.ContactType;
import org.briarproject.bramble.api.data.BdfReaderFactory;
import org.briarproject.bramble.api.data.BdfWriterFactory;
import org.briarproject.bramble.api.lifecycle.IoExecutor;
import org.briarproject.bramble.api.plugin.ConnectionManager;
import org.briarproject.bramble.api.sync.SyncSessionFactory;
import org.briarproject.bramble.api.transport.KeyManager;
import org.briarproject.bramble.api.transport.StreamReaderFactory;
import org.briarproject.bramble.api.transport.StreamWriterFactory;
import org.briarproject.bramble.mailbox.protocol.MailboxProtocol;

import java.util.concurrent.Executor;

import javax.inject.Inject;

public class MailboxSessionFactoryImpl implements MailboxSessionFactory {

	private Executor ioExecutor;
	private final KeyManager keyManager;
	private final SyncSessionFactory syncSessionFactory;
	private final StreamWriterFactory streamWriterFactory;
	private final StreamReaderFactory streamReaderFactory;
	private ConnectionManager connectionManager;
	private BdfWriterFactory bdfWriterFactory;
	private BdfReaderFactory bdfReaderFactory;

	@Inject
	public MailboxSessionFactoryImpl(@IoExecutor Executor ioExecutor,
			KeyManager keyManager,
			SyncSessionFactory syncSessionFactory,
			StreamWriterFactory streamWriterFactory,
			StreamReaderFactory streamReaderFactory) {
		this.ioExecutor = ioExecutor;
		this.keyManager = keyManager;
		this.syncSessionFactory = syncSessionFactory;
		this.streamWriterFactory = streamWriterFactory;
		this.streamReaderFactory = streamReaderFactory;
	}

	@Override
	public AbstractMailboxSession createMailboxSession(MailboxProtocol mailboxProtocol,
			ContactId contactId, ContactType contactType,
			int transportMaxLatency,
			int transportMaxIdleTime) {
		switch (contactType) {
			case CONTACT:
				break;
			case PRIVATE_MAILBOX:
				return new PrivateMailboxSession(contactId, ioExecutor,
						keyManager, syncSessionFactory, streamWriterFactory,
						streamReaderFactory, mailboxProtocol,
						transportMaxLatency, transportMaxIdleTime);
			case CONTACT_MAILBOX:
				return null;
			case MAILBOX_OWNER:
				return new MailboxOwnerSession(contactId, ioExecutor,
						keyManager, syncSessionFactory, streamWriterFactory,
						streamReaderFactory, mailboxProtocol,
						transportMaxLatency, transportMaxIdleTime);
		}
		return null;
	}
}
