package org.briarproject.bramble.mailbox.protocol;

import org.briarproject.bramble.api.FormatException;
import org.briarproject.bramble.api.contact.ContactId;
import org.briarproject.bramble.api.data.BdfList;

import static org.briarproject.bramble.mailbox.protocol.MailboxMessage.TYPE.STORE;

public class MailboxRequestStore extends MailboxRequest {
	private ContactId contactId;
	private byte[] encryptedSyncStream;

	public MailboxRequestStore(ContactId contactId,
			byte[] encryptedSyncMessage) {
		super(STORE);
		this.contactId = contactId;
		this.encryptedSyncStream = encryptedSyncMessage;
	}

	public MailboxRequestStore(BdfList msg) throws FormatException {
		super(msg);
	}

	public BdfList getRequestBody() {
		return BdfList.of(contactId.getInt(), encryptedSyncStream);
	}

	@Override
	public boolean hasResponse() {
		return true;
	}

	@Override
	public void parseBody(BdfList msg) throws FormatException {
		Long cId = msg.getLong(0);

		if (cId > Integer.MAX_VALUE || cId < Integer.MIN_VALUE)
			throw new FormatException();

		this.contactId = new ContactId(cId.intValue());
		this.encryptedSyncStream = msg.getRaw(1);
	}

	public byte[] getEncryptedSyncStream() {
		return encryptedSyncStream;
	}

	public ContactId getContactId() {
		return contactId;
	}
}
