package org.briarproject.bramble.mailbox.protocol;

import org.briarproject.bramble.api.FormatException;
import org.briarproject.bramble.api.data.BdfList;

import javax.annotation.Nullable;

import static org.briarproject.bramble.mailbox.protocol.MailboxMessage.TYPE.RESPONSE;

public class MailboxResponse implements MailboxMessage {


	private boolean success;
	private String errorMessage;
	private long msgId;

	public MailboxResponse(long msgId, boolean success,
			@Nullable String errorMessage) {
		this.msgId = msgId;
		this.success = success;
		this.errorMessage = errorMessage;
	}

	public MailboxResponse(BdfList msg) throws FormatException {
		this.msgId = msg.getLong(1);
		parseBody(msg.getList(2));
	}

	@Override
	public long getId() {
		return msgId;
	}

	@Override
	public boolean hasResponse() {
		return false;
	}

	@Override
	public TYPE getType() {
		return RESPONSE;
	}

	@Override
	public BdfList toBdfList() {
		BdfList body;
		if (!success)
			body = BdfList.of(success, errorMessage);
		else
			body = BdfList.of(success);

		return BdfList.of(RESPONSE.getValue(), msgId, body);
	}


	@Override
	public void parseBody(BdfList list) throws FormatException {
		success = list.getBoolean(0);
		if (!success)
			errorMessage = list.getString(1);
	}

	public String getErrorMessage() {
		return errorMessage;
	}

	public boolean isSuccess() {
		return success;
	}
}