package org.briarproject.briar.android.mailbox;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;

import org.briarproject.bramble.api.contact.ContactManager;
import org.briarproject.bramble.api.contact.PrivateMailbox;
import org.briarproject.bramble.api.contact.event.ContactRemovedEvent;
import org.briarproject.bramble.api.contact.event.ContactStatusChangedEvent;
import org.briarproject.bramble.api.db.DbException;
import org.briarproject.bramble.api.event.Event;
import org.briarproject.bramble.api.event.EventBus;
import org.briarproject.bramble.api.event.EventListener;
import org.briarproject.bramble.api.nullsafety.MethodsNotNullByDefault;
import org.briarproject.bramble.api.nullsafety.ParametersNotNullByDefault;
import org.briarproject.bramble.api.plugin.ConnectionRegistry;
import org.briarproject.bramble.api.plugin.event.ContactConnectedEvent;
import org.briarproject.bramble.api.plugin.event.ContactDisconnectedEvent;
import org.briarproject.briar.R;
import org.briarproject.briar.android.activity.ActivityComponent;
import org.briarproject.briar.android.fragment.BaseFragment;
import org.briarproject.briar.android.keyagreement.MailboxExchangeActivity;

import java.util.logging.Logger;

import javax.annotation.Nullable;
import javax.inject.Inject;

import static android.view.View.GONE;
import static android.view.View.VISIBLE;
import static java.util.logging.Level.WARNING;
import static org.briarproject.bramble.util.LogUtils.logException;

@MethodsNotNullByDefault
@ParametersNotNullByDefault
public class MailboxFragment extends BaseFragment implements EventListener {

	public static final String TAG = MailboxFragment.class.getName();
	private static final Logger LOG = Logger.getLogger(TAG);

	@Inject
	ContactManager contactManager;
	@Inject
	ConnectionRegistry connectionRegistry;
	@Inject
	EventBus eventBus;

	private TextView mailboxStatus;
	private Button deleteMailbox;

	public static MailboxFragment newInstance() {
		Bundle args = new Bundle();
		MailboxFragment fragment = new MailboxFragment();
		fragment.setArguments(args);
		return fragment;
	}

	@Override
	public String getUniqueTag() {
		return TAG;
	}

	@Override
	public void injectFragment(ActivityComponent component) {
		component.inject(this);
	}

	@Nullable
	@Override
	public View onCreateView(LayoutInflater inflater,
			@Nullable ViewGroup container,
			@Nullable Bundle savedInstanceState) {

		getActivity().setTitle(R.string.mailbox);
		View contentView =
				inflater.inflate(R.layout.fragment_mailbox, container, false);
		mailboxStatus = contentView.findViewById(R.id.mailboxStatus);
		deleteMailbox = contentView.findViewById(R.id.deleteButton);
		return contentView;
	}

	@Override
	public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
		inflater.inflate(R.menu.contact_list_actions, menu);
		super.onCreateOptionsMenu(menu, inflater);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// Handle presses on the action bar items
		switch (item.getItemId()) {
			case R.id.action_add_contact:
				Intent intent =
						new Intent(getContext(), MailboxExchangeActivity.class);
				startActivity(intent);
				return true;
			default:
				return super.onOptionsItemSelected(item);
		}
	}

	@Override
	public void onStart() {
		super.onStart();
		eventBus.addListener(this);
		loadPrivateMailbox();
	}

	@Override
	public void onStop() {
		super.onStop();
		eventBus.removeListener(this);
	}

	private void loadPrivateMailbox() {
		listener.runOnDbThread(() -> {
			try {
				PrivateMailbox privateMailbox =
						contactManager.getPrivateMailbox();
				runOnUiThreadUnlessDestroyed(() -> {
					if (privateMailbox == null) {
						mailboxStatus.setText(R.string.mailbox_unpaired);
						deleteMailbox.setVisibility(GONE);
					} else {
						deleteMailbox.setVisibility(VISIBLE);
						boolean connected =
								connectionRegistry
										.isConnected(privateMailbox.getId());
						if (connected)
							mailboxStatus.setText(R.string.mailbox_connected);
						else
							mailboxStatus
									.setText(R.string.mailbox_disconnected);
					}
				});
			} catch (DbException e) {
				logException(LOG, WARNING, e);
			}
		});
	}

	@Override
	public void eventOccurred(Event e) {
		if (e instanceof ContactStatusChangedEvent) {
			ContactStatusChangedEvent c = (ContactStatusChangedEvent) e;
			if (c.isActive()) {
				LOG.info("Contact activated, reloading");
			} else {
				LOG.info("Contact deactivated, removing item");
			}
		} else if (e instanceof ContactConnectedEvent) {
		} else if (e instanceof ContactDisconnectedEvent) {
		} else if (e instanceof ContactRemovedEvent) {
			LOG.info("Contact removed, removing item");
		}
	}

}
