package org.briarproject.briar.android.repeater;


import android.os.Bundle;
import android.support.v7.widget.Toolbar;
import android.widget.TextView;

import org.briarproject.bramble.api.contact.Contact;
import org.briarproject.bramble.api.contact.ContactManager;
import org.briarproject.bramble.api.db.DatabaseExecutor;
import org.briarproject.bramble.api.db.DbException;
import org.briarproject.briar.R;
import org.briarproject.briar.android.activity.ActivityComponent;
import org.briarproject.briar.android.activity.BriarActivity;

import java.util.Collection;
import java.util.concurrent.Executor;
import java.util.logging.Logger;

import javax.annotation.Nullable;
import javax.inject.Inject;

import de.hdodenhof.circleimageview.CircleImageView;

import static android.support.v4.view.ViewCompat.setTransitionName;
import static java.util.logging.Level.WARNING;
import static org.briarproject.bramble.api.contact.ContactType.PRIVATE_MAILBOX;
import static org.briarproject.bramble.util.LogUtils.logException;

public class RepeaterActivity extends BriarActivity {

	private static final Logger LOG =
			Logger.getLogger(RepeaterActivity.class.getName());
	@Inject
	@DatabaseExecutor
	Executor databaseExecutor;
	@Inject
	ContactManager contactManager;

	private Toolbar toolbar;
	private CircleImageView toolbarAvatar;
	private TextView toolbarTitle;

	@SuppressWarnings("ConstantConditions")
	@Override
	public void onCreate(@Nullable Bundle state) {
		setSceneTransitionAnimation();
		super.onCreate(state);

		setContentView(R.layout.activity_repeater);
		// Custom Toolbar
		toolbar = setUpCustomToolbar(true);
		if (toolbar != null) {
			toolbarAvatar = toolbar.findViewById(R.id.contactAvatar);
			toolbarTitle = toolbar.findViewById(R.id.repeaterName);
		}

		setTransitionName(toolbarAvatar, "avatarRepeater");
	}

	@Override
	public void onStart() {
		super.onStart();
		loadRepeater();
	}

	private void loadRepeater() {
		databaseExecutor.execute(() -> {
			try {
				Collection<Contact> repeaters =
						contactManager.getContactsByType(PRIVATE_MAILBOX);
				if (repeaters.size() > 1)
					throw new IllegalStateException(
							"More than one repeater found");
				if (repeaters.isEmpty())
					displayRepeater(null);
				displayRepeater(repeaters.iterator().next());
			} catch (DbException e) {
				logException(LOG, WARNING, e);
			}
		});
	}

	private void displayRepeater(@Nullable Contact repeater) {
		runOnUiThreadUnlessDestroyed(() -> {
			// TODO: Repeater Image
			toolbarAvatar.setImageResource(R.mipmap.ic_launcher_round);
			toolbarTitle.setText(repeater.getAuthor().getName());
		});
	}

	@Override
	public void injectActivity(ActivityComponent component) {
		component.inject(this);
	}
}
